#ifndef __IUF_GAMEOBJ
#define __IUF_GAMEOBJ
#include <math.h>
#include <vcg/Point3.h>
#include <vcg/Sphere3.h>
#include <vector>
#include <list>

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

/** This class represent a generic game object, with its speed, position etc.. 
This class is the base class  that is used to derive all the objects represented 
in the game: asteroids, ships bullets explosion and so on. 
Each GameObj has a specific position in the game space, a 
velocity vector, a heading (direction) angle (that can be 
obviously different from the velocity direction), a main 
rotation axis and an angular speed (think of asteroids inertia).

Version History {\small 
\begin{itemize}
\item 0.1 First Release.
\item 0.3 Added Bounding Sphere B for collision detection 
management and Collide() function; 
This function is called in the \Ref{GameSession} class.
\item 0.4 Changed return value of draw from void to int: 
now it return the number of drawn triangles
\end{itemize}
}
*/
using namespace vcg;
using namespace std;
class GameObj
{
 public:
	/// Default constructor
	GameObj();
	/// constructor with given position and direction
	GameObj(const Point3f &_p, const float _angle);
	/// constructor with given position and direction and velocity
	GameObj(const Point3f &_p, const float _angle, const Point3f &_v);
	/// Position of a generic object in the game
	Point3f p;	  
	/// Rotation Axis 
	Point3f axis;	  
	///Direction (in degree)
	float angle;  
	/// Speed Component (mt / sec)  
	Point3f v;  
	/// Angular Speed (deg / sec)
	float av;
	/// If the object is active or not
	bool active; 

	/// Bounding sphere of the object
	Sphere3f B;

	/** Basic Redraw Function it does nothing.
	All derived classes should provide their own Draw functions.
	Version History {\small 
	\begin{itemize}
	\item 0.1 First Release.
	\item 0.4 Changed return value from void to int: 
	now it return the number of drawn triangles
	\end{itemize}
	}
	*/
	virtual int Draw(){return 0;}
	/** Basic update function.
	It updates the position and the direction of the object according its current 
	speed and the elapsed time (expressed as millisec).
	*/
	virtual void Update(int t)
		{
			p += (t/1000.0f*v);
			angle += (t/1000.0f*av);
		};
	/** Return the unitary direction vector.
	*/

	Point3f VDir() {
		return Point3f(cos(M_PI*angle/180.0),
									 sin(M_PI*angle/180.0),
									 0);}; 

/** This function check if the current 
object collide with the passed one.
The  two objects collide if their bounding 
spheres has an intersection.
Note that this is just a rough approximation of a real collision
detetction. A much better solution could be obtained by substituting 
spheres with hierarchies of bounding objects and testing the 
intersection between the various levels of the hierarchies. Perhaps in next versions...
*/
  virtual bool Collide(GameObj *o);

/** Utility function that draw the bounding sphere */
	void DrawBound();

 /// Less operator needed for stl containers
 inline bool operator <  ( GameObj const & o ) const {
			return	p<o.p;
 }
 /// Equal operator needed for some stl containers.
 // Not very useful as it is written now...
 inline bool operator == ( GameObj const & o ) const {
	return (p==o.p && v==o.v);
 }

};

#endif // ifndef __IUF_GAMEOBJ	